import { __decorate, __param } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Inject, ElementRef, Renderer2, NgZone, Input, Output, Directive, NgModule } from '@angular/core';
import { WINDOW } from 'ngx-window-token';
import { Subject, merge, fromEvent } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

let SpotlightService = class SpotlightService {
    constructor() {
        /** available spotlights list */
        this._targetMap = new Map();
    }
    /**
     * Registers the spotlight directive into the service
     * @param id - spotlight id
     * @param target - spotlight's directive reference
     */
    register(id, target) {
        if (this._targetMap.has(id)) {
            throw new Error(`The ${id} spotlight is already registered!`);
        }
        this._targetMap.set(id, target);
    }
    /**
     * All registred directive ids
     */
    getIdlist() {
        return Array.from(this._targetMap.keys());
    }
    /**
     * Get SpotlightDirective instance by id
     * @param id - spotlight id
     */
    getById(id) {
        return this._targetMap.get(id);
    }
    /**
     * Unregisters the spotlight directive from the service. Should be used in directive's `OnDestroy` hook
     * @param id - spotlight's id
     */
    deregister(id) {
        this._targetMap.delete(id);
    }
};
SpotlightService.ɵprov = ɵɵdefineInjectable({ factory: function SpotlightService_Factory() { return new SpotlightService(); }, token: SpotlightService, providedIn: "root" });
SpotlightService = __decorate([
    Injectable({
        providedIn: 'root',
    })
], SpotlightService);

function capitalize(value) {
    return value.charAt(0).toUpperCase() + value.slice(1);
}
/**
 * Modifies the given layer's styles **by mutating it**
 * @param el - backdrop's layer HTML element
 * @param piece - piece of spotlight element
 */
function getStyle(rects, piece, borderWidth, indent) {
    const left = rects.left - indent;
    const top = rects.top - indent;
    const right = rects.right + indent;
    const bottom = rects.bottom + indent;
    const width = rects.width + 2 * indent;
    const height = rects.height + 2 * indent;
    switch (piece) {
        case 'backdrop-top':
            return {
                top: '0',
                left: '0',
                right: '0',
                height: `${top}px`,
            };
        case 'backdrop-bottom':
            return {
                top: `${bottom}px`,
                left: '0',
                right: '0',
                bottom: '0',
            };
        case 'backdrop-left':
            return {
                top: `${top}px`,
                left: '0',
                width: `${left}px`,
                height: `${height}px`,
            };
        case 'backdrop-right':
            return {
                top: `${top}px`,
                left: `${right}px`,
                right: '0',
                height: `${height}px`,
            };
        case 'overlay':
            return {
                top: `${top}px`,
                left: `${left}px`,
                width: `${width}px`,
                height: `${height}px`,
                position: 'fixed',
                zIndex: '990',
            };
        case 'border-top':
            return {
                borderTopStyle: 'solid',
                top: `${top - borderWidth}px`,
                left: `${left - borderWidth}px`,
                height: `${borderWidth}px`,
                width: `${width + 2 * borderWidth}px`,
            };
        case 'border-bottom':
            return {
                top: `${bottom - borderWidth}px`,
                left: `${left - borderWidth}px`,
                height: `${borderWidth}px`,
                width: `${width + 2 * borderWidth}px`,
            };
        case 'border-left':
            return {
                top: `${top - borderWidth}px`,
                left: `${left - borderWidth}px`,
                height: `${height + 2 * borderWidth}px`,
                width: `${borderWidth}px`,
            };
        case 'border-right':
            return {
                top: `${top - borderWidth}px`,
                left: `${right - borderWidth}px`,
                height: `${height + 2 * borderWidth}px`,
                width: `${borderWidth}px`,
            };
        default:
            throw new Error(`Unexpected element ${piece}`);
    }
}

/**
 * Spotlight directive
 *
 * This lets you to spotlight certain elements
 * by placing 4 dark transparent divs around it
 */
let SpotlightDirective = class SpotlightDirective {
    constructor(_spotlightService, _window, elementRef, _renderer, _zone) {
        this._spotlightService = _spotlightService;
        this.elementRef = elementRef;
        this._renderer = _renderer;
        this._zone = _zone;
        /** spotlight id */
        this.id = 'spotlight_at_' + Date.now();
        /** draw border around spotlight element */
        this.border = false;
        /** space around spotlight element */
        this.indent = 0;
        /** draw transparent overlay on spotlight element */
        this.overlay = false;
        /** highlight element after view init */
        this.auto = false;
        /** border width */
        this.borderWidth = 4;
        /** backdrop and overlay click event */
        this.spotlightClick = new EventEmitter();
        /** whether the directive is shown */
        this._isShown = false;
        /** destroy subject (pattern) */
        this._destroy$ = new Subject();
        /** set of all created corresponding elements (backdrops, borders and overlay) */
        this._elementMap = new Map();
        /**
         * Set of listeners that should be "unlistened" on destroying
         * @see https://angular.io/api/core/Renderer2#listen
         */
        this._listenerSet = new Set();
        this._window = _window;
    }
    /** whether the spotlight is shown */
    get isShown() {
        return this._isShown;
    }
    /** main spotlight container */
    get container() {
        return this._elementMap.get('container');
    }
    ngAfterViewInit() {
        this._spotlightService.register(this.id, this);
        if (this.auto) {
            this.show();
        }
    }
    /**
     * Shows the overlay
     * - Scrolls the page to the very top
     * - Places 4 overlays around the element
     */
    show() {
        if (this._isShown) {
            return;
        }
        this._drawContainer();
        this._drawBackdrop();
        if (this.border) {
            this._drawBorder();
        }
        if (this.overlay) {
            this._drawOverlay();
        }
        this._watchWindowUpdate();
        this._isShown = true;
    }
    /**
     * Places container to document body
     */
    _drawContainer() {
        const container = this._renderer.createElement('div');
        this._renderer.addClass(container, 'spotlight__container'); // just for show class
        this._setStyles(container, {
            position: 'absolute',
            zIndex: '940',
            left: '0px',
            top: '0px',
            right: '0px',
            bottom: '0px',
            pointerEvents: 'none',
        });
        this._elementMap.set('container', container);
        this._renderer.appendChild(this._window.document.body, container);
    }
    /**
     * Places 4 backdrops around the elementRef
     */
    _drawBackdrop() {
        for (const side of ['top', 'bottom', 'left', 'right']) {
            const backdrop = `backdrop-${side}`;
            const backdropEl = this._renderer.createElement('div');
            this._renderer.addClass(backdropEl, 'spotlight__backdrop'); // just for show class
            this._renderer.addClass(backdropEl, `spotlight__backdrop_${side}`); // just for show class
            this._setStyles(backdropEl, {
                position: 'fixed',
                zIndex: '950',
                display: 'block',
                backgroundColor: `var(
          --color__spotlight-backdrop_background,
          rgba(52, 74, 94, 0.8)
        )`,
            });
            this._modifyPieceStyle(backdropEl, backdrop);
            this._listenerSet.add(this._renderer.listen(backdropEl, 'click', (event) => {
                event.preventDefault();
                this.spotlightClick.emit({ piece: backdrop, mouse: event });
            }));
            this._elementMap.set(backdrop, backdropEl);
            this._renderer.appendChild(this.container, backdropEl);
        }
    }
    /**
     * Places 4 div around the spotlight to emulate stroke
     */
    _drawBorder() {
        for (const side of ['top', 'bottom', 'left', 'right']) {
            const border = `border-${side}`;
            const borderEl = this._renderer.createElement('div');
            this._renderer.addClass(borderEl, 'spotlight__border'); // just for show class
            this._renderer.addClass(borderEl, `spotlight__border_${side}`); // just for show class
            this._setStyles(borderEl, {
                position: 'fixed',
                zIndex: '955',
                pointerEvents: 'none',
                borderRadius: '20px',
                display: 'block',
                borderColor: `var(
          --color__spotlight-border,
          #c9c9c9
        )`,
                [`border${capitalize(side)}Width`]: this.borderWidth + 'px',
                [`border${capitalize(side)}Style`]: 'solid',
            });
            this._modifyPieceStyle(borderEl, border);
            this._elementMap.set(border, borderEl);
            this._renderer.appendChild(this.container, borderEl);
        }
    }
    /**
     * Watches for window scroll&resize and, when it occurs, modifies the layers styles
     */
    _watchWindowUpdate() {
        const updateHtmlElements = () => {
            for (const [piece, layer] of this._elementMap) {
                this._modifyPieceStyle(layer, piece);
            }
        };
        this._zone.runOutsideAngular(() => {
            merge(fromEvent(window, 'scroll'), fromEvent(window, 'resize'))
                .pipe(takeUntil(this._destroy$))
                .subscribe(() => updateHtmlElements());
        });
    }
    /**
     * Sets backdrop piece styles
     * @param element - the element to mutate
     * @param props - the limited set of CSS properties
     */
    _setStyles(element, props) {
        for (const prop in props) {
            if (props.hasOwnProperty(prop)) {
                this._renderer.setStyle(element, prop, props[prop]);
            }
        }
    }
    /**
     * Modifies the given layer's styles **by mutating it**
     * @param el - backdrop's layer HTML element
     * @param piece - piece of spotlight element
     */
    _modifyPieceStyle(el, piece) {
        if (piece !== 'container') {
            const rects = this.elementRef.nativeElement.getBoundingClientRect();
            const style = getStyle(rects, piece, this.borderWidth, this.indent);
            this._setStyles(el, style);
        }
    }
    /**
     * Draws an invisible element over the spotlighted element from the DOM
     */
    _drawOverlay() {
        const overlay = this._renderer.createElement('div');
        this._renderer.addClass(overlay, 'spotlight__cover'); // just for show element
        this._modifyPieceStyle(overlay, 'overlay');
        this._listenerSet.add(this._renderer.listen(overlay, 'click', (event) => {
            event.preventDefault();
            this.spotlightClick.emit({ piece: 'overlay', mouse: event });
        }));
        this._elementMap.set('overlay', overlay);
        this._renderer.appendChild(this.elementRef.nativeElement.parentElement, overlay);
    }
    /**
     * Removes the spotlight for this element
     */
    hide() {
        if (!this._isShown) {
            return;
        }
        this._removeElements();
        this._destroy$.next();
        this._isShown = false;
    }
    /**
     * Removes the backdrop and the overlay elements from the DOM
     */
    _removeElements() {
        for (const removeListener of this._listenerSet) {
            removeListener();
        }
        this._listenerSet.clear();
        for (const layer of this._elementMap.values()) {
            layer.remove();
        }
        this._elementMap.clear();
    }
    ngOnDestroy() {
        this._destroy$.next();
        this._destroy$.complete();
        this.hide();
        this._spotlightService.deregister(this.id);
    }
};
SpotlightDirective.ctorParameters = () => [
    { type: SpotlightService },
    { type: undefined, decorators: [{ type: Inject, args: [WINDOW,] }] },
    { type: ElementRef },
    { type: Renderer2 },
    { type: NgZone }
];
__decorate([
    Input('ngxSpotlight')
], SpotlightDirective.prototype, "id", void 0);
__decorate([
    Input()
], SpotlightDirective.prototype, "border", void 0);
__decorate([
    Input()
], SpotlightDirective.prototype, "indent", void 0);
__decorate([
    Input()
], SpotlightDirective.prototype, "overlay", void 0);
__decorate([
    Input()
], SpotlightDirective.prototype, "auto", void 0);
__decorate([
    Input()
], SpotlightDirective.prototype, "borderWidth", void 0);
__decorate([
    Output()
], SpotlightDirective.prototype, "spotlightClick", void 0);
SpotlightDirective = __decorate([
    Directive({
        selector: '[ngxSpotlight]',
        exportAs: 'spotlight',
    }),
    __param(1, Inject(WINDOW))
], SpotlightDirective);

/**
 * Spotlight directive.
 * It's used to highlight specific element on the page
 *
 * Usage:
 * ```html
 * <some-component ngxSpotlight></some-component>
 * ```
 */
let NgxSpotlightModule = class NgxSpotlightModule {
};
NgxSpotlightModule = __decorate([
    NgModule({
        declarations: [SpotlightDirective],
        imports: [CommonModule],
        exports: [SpotlightDirective]
    })
], NgxSpotlightModule);

/*
 * Public API Surface of spotlight
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxSpotlightModule, SpotlightDirective, SpotlightService };
//# sourceMappingURL=ngx-spotlight.js.map
