import { __decorate, __values, __read, __param } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Inject, ElementRef, Renderer2, NgZone, Input, Output, Directive, NgModule } from '@angular/core';
import { WINDOW } from 'ngx-window-token';
import { Subject, merge, fromEvent } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

var SpotlightService = /** @class */ (function () {
    function SpotlightService() {
        /** available spotlights list */
        this._targetMap = new Map();
    }
    /**
     * Registers the spotlight directive into the service
     * @param id - spotlight id
     * @param target - spotlight's directive reference
     */
    SpotlightService.prototype.register = function (id, target) {
        if (this._targetMap.has(id)) {
            throw new Error("The " + id + " spotlight is already registered!");
        }
        this._targetMap.set(id, target);
    };
    /**
     * All registred directive ids
     */
    SpotlightService.prototype.getIdlist = function () {
        return Array.from(this._targetMap.keys());
    };
    /**
     * Get SpotlightDirective instance by id
     * @param id - spotlight id
     */
    SpotlightService.prototype.getById = function (id) {
        return this._targetMap.get(id);
    };
    /**
     * Unregisters the spotlight directive from the service. Should be used in directive's `OnDestroy` hook
     * @param id - spotlight's id
     */
    SpotlightService.prototype.deregister = function (id) {
        this._targetMap.delete(id);
    };
    SpotlightService.ɵprov = ɵɵdefineInjectable({ factory: function SpotlightService_Factory() { return new SpotlightService(); }, token: SpotlightService, providedIn: "root" });
    SpotlightService = __decorate([
        Injectable({
            providedIn: 'root',
        })
    ], SpotlightService);
    return SpotlightService;
}());

function capitalize(value) {
    return value.charAt(0).toUpperCase() + value.slice(1);
}
/**
 * Modifies the given layer's styles **by mutating it**
 * @param el - backdrop's layer HTML element
 * @param piece - piece of spotlight element
 */
function getStyle(rects, piece, borderWidth, indent) {
    var left = rects.left - indent;
    var top = rects.top - indent;
    var right = rects.right + indent;
    var bottom = rects.bottom + indent;
    var width = rects.width + 2 * indent;
    var height = rects.height + 2 * indent;
    switch (piece) {
        case 'backdrop-top':
            return {
                top: '0',
                left: '0',
                right: '0',
                height: top + "px",
            };
        case 'backdrop-bottom':
            return {
                top: bottom + "px",
                left: '0',
                right: '0',
                bottom: '0',
            };
        case 'backdrop-left':
            return {
                top: top + "px",
                left: '0',
                width: left + "px",
                height: height + "px",
            };
        case 'backdrop-right':
            return {
                top: top + "px",
                left: right + "px",
                right: '0',
                height: height + "px",
            };
        case 'overlay':
            return {
                top: top + "px",
                left: left + "px",
                width: width + "px",
                height: height + "px",
                position: 'fixed',
                zIndex: '990',
            };
        case 'border-top':
            return {
                borderTopStyle: 'solid',
                top: top - borderWidth + "px",
                left: left - borderWidth + "px",
                height: borderWidth + "px",
                width: width + 2 * borderWidth + "px",
            };
        case 'border-bottom':
            return {
                top: bottom - borderWidth + "px",
                left: left - borderWidth + "px",
                height: borderWidth + "px",
                width: width + 2 * borderWidth + "px",
            };
        case 'border-left':
            return {
                top: top - borderWidth + "px",
                left: left - borderWidth + "px",
                height: height + 2 * borderWidth + "px",
                width: borderWidth + "px",
            };
        case 'border-right':
            return {
                top: top - borderWidth + "px",
                left: right - borderWidth + "px",
                height: height + 2 * borderWidth + "px",
                width: borderWidth + "px",
            };
        default:
            throw new Error("Unexpected element " + piece);
    }
}

/**
 * Spotlight directive
 *
 * This lets you to spotlight certain elements
 * by placing 4 dark transparent divs around it
 */
var SpotlightDirective = /** @class */ (function () {
    function SpotlightDirective(_spotlightService, _window, elementRef, _renderer, _zone) {
        this._spotlightService = _spotlightService;
        this.elementRef = elementRef;
        this._renderer = _renderer;
        this._zone = _zone;
        /** spotlight id */
        this.id = 'spotlight_at_' + Date.now();
        /** draw border around spotlight element */
        this.border = false;
        /** space around spotlight element */
        this.indent = 0;
        /** draw transparent overlay on spotlight element */
        this.overlay = false;
        /** highlight element after view init */
        this.auto = false;
        /** border width */
        this.borderWidth = 4;
        /** backdrop and overlay click event */
        this.spotlightClick = new EventEmitter();
        /** whether the directive is shown */
        this._isShown = false;
        /** destroy subject (pattern) */
        this._destroy$ = new Subject();
        /** set of all created corresponding elements (backdrops, borders and overlay) */
        this._elementMap = new Map();
        /**
         * Set of listeners that should be "unlistened" on destroying
         * @see https://angular.io/api/core/Renderer2#listen
         */
        this._listenerSet = new Set();
        this._window = _window;
    }
    Object.defineProperty(SpotlightDirective.prototype, "isShown", {
        /** whether the spotlight is shown */
        get: function () {
            return this._isShown;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SpotlightDirective.prototype, "container", {
        /** main spotlight container */
        get: function () {
            return this._elementMap.get('container');
        },
        enumerable: true,
        configurable: true
    });
    SpotlightDirective.prototype.ngAfterViewInit = function () {
        this._spotlightService.register(this.id, this);
        if (this.auto) {
            this.show();
        }
    };
    /**
     * Shows the overlay
     * - Scrolls the page to the very top
     * - Places 4 overlays around the element
     */
    SpotlightDirective.prototype.show = function () {
        if (this._isShown) {
            return;
        }
        this._drawContainer();
        this._drawBackdrop();
        if (this.border) {
            this._drawBorder();
        }
        if (this.overlay) {
            this._drawOverlay();
        }
        this._watchWindowUpdate();
        this._isShown = true;
    };
    /**
     * Places container to document body
     */
    SpotlightDirective.prototype._drawContainer = function () {
        var container = this._renderer.createElement('div');
        this._renderer.addClass(container, 'spotlight__container'); // just for show class
        this._setStyles(container, {
            position: 'absolute',
            zIndex: '940',
            left: '0px',
            top: '0px',
            right: '0px',
            bottom: '0px',
            pointerEvents: 'none',
        });
        this._elementMap.set('container', container);
        this._renderer.appendChild(this._window.document.body, container);
    };
    /**
     * Places 4 backdrops around the elementRef
     */
    SpotlightDirective.prototype._drawBackdrop = function () {
        var e_1, _a;
        var _this = this;
        var _loop_1 = function (side) {
            var backdrop = "backdrop-" + side;
            var backdropEl = this_1._renderer.createElement('div');
            this_1._renderer.addClass(backdropEl, 'spotlight__backdrop'); // just for show class
            this_1._renderer.addClass(backdropEl, "spotlight__backdrop_" + side); // just for show class
            this_1._setStyles(backdropEl, {
                position: 'fixed',
                zIndex: '950',
                display: 'block',
                backgroundColor: "var(\n          --color__spotlight-backdrop_background,\n          rgba(52, 74, 94, 0.8)\n        )",
            });
            this_1._modifyPieceStyle(backdropEl, backdrop);
            this_1._listenerSet.add(this_1._renderer.listen(backdropEl, 'click', function (event) {
                event.preventDefault();
                _this.spotlightClick.emit({ piece: backdrop, mouse: event });
            }));
            this_1._elementMap.set(backdrop, backdropEl);
            this_1._renderer.appendChild(this_1.container, backdropEl);
        };
        var this_1 = this;
        try {
            for (var _b = __values(['top', 'bottom', 'left', 'right']), _c = _b.next(); !_c.done; _c = _b.next()) {
                var side = _c.value;
                _loop_1(side);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * Places 4 div around the spotlight to emulate stroke
     */
    SpotlightDirective.prototype._drawBorder = function () {
        var e_2, _a, _b;
        try {
            for (var _c = __values(['top', 'bottom', 'left', 'right']), _d = _c.next(); !_d.done; _d = _c.next()) {
                var side = _d.value;
                var border = "border-" + side;
                var borderEl = this._renderer.createElement('div');
                this._renderer.addClass(borderEl, 'spotlight__border'); // just for show class
                this._renderer.addClass(borderEl, "spotlight__border_" + side); // just for show class
                this._setStyles(borderEl, (_b = {
                        position: 'fixed',
                        zIndex: '955',
                        pointerEvents: 'none',
                        borderRadius: '20px',
                        display: 'block',
                        borderColor: "var(\n          --color__spotlight-border,\n          #c9c9c9\n        )"
                    },
                    _b["border" + capitalize(side) + "Width"] = this.borderWidth + 'px',
                    _b["border" + capitalize(side) + "Style"] = 'solid',
                    _b));
                this._modifyPieceStyle(borderEl, border);
                this._elementMap.set(border, borderEl);
                this._renderer.appendChild(this.container, borderEl);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    /**
     * Watches for window scroll&resize and, when it occurs, modifies the layers styles
     */
    SpotlightDirective.prototype._watchWindowUpdate = function () {
        var _this = this;
        var updateHtmlElements = function () {
            var e_3, _a;
            try {
                for (var _b = __values(_this._elementMap), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = __read(_c.value, 2), piece = _d[0], layer = _d[1];
                    _this._modifyPieceStyle(layer, piece);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
        };
        this._zone.runOutsideAngular(function () {
            merge(fromEvent(window, 'scroll'), fromEvent(window, 'resize'))
                .pipe(takeUntil(_this._destroy$))
                .subscribe(function () { return updateHtmlElements(); });
        });
    };
    /**
     * Sets backdrop piece styles
     * @param element - the element to mutate
     * @param props - the limited set of CSS properties
     */
    SpotlightDirective.prototype._setStyles = function (element, props) {
        for (var prop in props) {
            if (props.hasOwnProperty(prop)) {
                this._renderer.setStyle(element, prop, props[prop]);
            }
        }
    };
    /**
     * Modifies the given layer's styles **by mutating it**
     * @param el - backdrop's layer HTML element
     * @param piece - piece of spotlight element
     */
    SpotlightDirective.prototype._modifyPieceStyle = function (el, piece) {
        if (piece !== 'container') {
            var rects = this.elementRef.nativeElement.getBoundingClientRect();
            var style = getStyle(rects, piece, this.borderWidth, this.indent);
            this._setStyles(el, style);
        }
    };
    /**
     * Draws an invisible element over the spotlighted element from the DOM
     */
    SpotlightDirective.prototype._drawOverlay = function () {
        var _this = this;
        var overlay = this._renderer.createElement('div');
        this._renderer.addClass(overlay, 'spotlight__cover'); // just for show element
        this._modifyPieceStyle(overlay, 'overlay');
        this._listenerSet.add(this._renderer.listen(overlay, 'click', function (event) {
            event.preventDefault();
            _this.spotlightClick.emit({ piece: 'overlay', mouse: event });
        }));
        this._elementMap.set('overlay', overlay);
        this._renderer.appendChild(this.elementRef.nativeElement.parentElement, overlay);
    };
    /**
     * Removes the spotlight for this element
     */
    SpotlightDirective.prototype.hide = function () {
        if (!this._isShown) {
            return;
        }
        this._removeElements();
        this._destroy$.next();
        this._isShown = false;
    };
    /**
     * Removes the backdrop and the overlay elements from the DOM
     */
    SpotlightDirective.prototype._removeElements = function () {
        var e_4, _a, e_5, _b;
        try {
            for (var _c = __values(this._listenerSet), _d = _c.next(); !_d.done; _d = _c.next()) {
                var removeListener = _d.value;
                removeListener();
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_4) throw e_4.error; }
        }
        this._listenerSet.clear();
        try {
            for (var _e = __values(this._elementMap.values()), _f = _e.next(); !_f.done; _f = _e.next()) {
                var layer = _f.value;
                layer.remove();
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
            }
            finally { if (e_5) throw e_5.error; }
        }
        this._elementMap.clear();
    };
    SpotlightDirective.prototype.ngOnDestroy = function () {
        this._destroy$.next();
        this._destroy$.complete();
        this.hide();
        this._spotlightService.deregister(this.id);
    };
    SpotlightDirective.ctorParameters = function () { return [
        { type: SpotlightService },
        { type: undefined, decorators: [{ type: Inject, args: [WINDOW,] }] },
        { type: ElementRef },
        { type: Renderer2 },
        { type: NgZone }
    ]; };
    __decorate([
        Input('ngxSpotlight')
    ], SpotlightDirective.prototype, "id", void 0);
    __decorate([
        Input()
    ], SpotlightDirective.prototype, "border", void 0);
    __decorate([
        Input()
    ], SpotlightDirective.prototype, "indent", void 0);
    __decorate([
        Input()
    ], SpotlightDirective.prototype, "overlay", void 0);
    __decorate([
        Input()
    ], SpotlightDirective.prototype, "auto", void 0);
    __decorate([
        Input()
    ], SpotlightDirective.prototype, "borderWidth", void 0);
    __decorate([
        Output()
    ], SpotlightDirective.prototype, "spotlightClick", void 0);
    SpotlightDirective = __decorate([
        Directive({
            selector: '[ngxSpotlight]',
            exportAs: 'spotlight',
        }),
        __param(1, Inject(WINDOW))
    ], SpotlightDirective);
    return SpotlightDirective;
}());

/**
 * Spotlight directive.
 * It's used to highlight specific element on the page
 *
 * Usage:
 * ```html
 * <some-component ngxSpotlight></some-component>
 * ```
 */
var NgxSpotlightModule = /** @class */ (function () {
    function NgxSpotlightModule() {
    }
    NgxSpotlightModule = __decorate([
        NgModule({
            declarations: [SpotlightDirective],
            imports: [CommonModule],
            exports: [SpotlightDirective]
        })
    ], NgxSpotlightModule);
    return NgxSpotlightModule;
}());

/*
 * Public API Surface of spotlight
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxSpotlightModule, SpotlightDirective, SpotlightService };
//# sourceMappingURL=ngx-spotlight.js.map
